@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2003-2007  The Chemistry Development Kit (CDK) project
  *                    2014  Mark B Vine (orcid:0000-0002-7794-0426)
  *
@@ -29,6 +29,7 @@ import org.junit.Assert;
 import org.junit.BeforeClass;
 import org.junit.Test;
 import org.openscience.cdk.AtomContainer;
+import org.openscience.cdk.AtomRef;
 import org.openscience.cdk.CDKConstants;
 import org.openscience.cdk.ChemFile;
 import org.openscience.cdk.ChemModel;
@@ -41,7 +42,6 @@ import org.openscience.cdk.interfaces.IAtomContainerSet;
 import org.openscience.cdk.interfaces.IBond;
 import org.openscience.cdk.interfaces.IBond.Order;
 import org.openscience.cdk.interfaces.IChemFile;
-import org.openscience.cdk.interfaces.IChemObject;
 import org.openscience.cdk.interfaces.IChemObjectBuilder;
 import org.openscience.cdk.interfaces.IPseudoAtom;
 import org.openscience.cdk.interfaces.IStereoElement;
@@ -49,6 +49,9 @@ import org.openscience.cdk.interfaces.ITetrahedralChirality;
 import org.openscience.cdk.io.IChemObjectReader.Mode;
 import org.openscience.cdk.io.listener.PropertiesListener;
 import org.openscience.cdk.isomorphism.matchers.Expr;
+import org.openscience.cdk.isomorphism.matchers.IQueryAtomContainer;
+import org.openscience.cdk.isomorphism.matchers.QueryAtom;
+import org.openscience.cdk.isomorphism.matchers.QueryAtomContainer;
 import org.openscience.cdk.isomorphism.matchers.QueryBond;
 import org.openscience.cdk.sgroup.Sgroup;
 import org.openscience.cdk.sgroup.SgroupBracket;
@@ -65,6 +68,7 @@ import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.StringReader;
+import java.io.StringWriter;
 import java.nio.charset.StandardCharsets;
 import java.util.Iterator;
 import java.util.List;
@@ -72,10 +76,18 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
 
-import static org.hamcrest.CoreMatchers.*;
+import static org.hamcrest.CoreMatchers.containsString;
+import static org.hamcrest.CoreMatchers.hasItem;
+import static org.hamcrest.CoreMatchers.instanceOf;
+import static org.hamcrest.CoreMatchers.is;
+import static org.hamcrest.CoreMatchers.not;
+import static org.hamcrest.CoreMatchers.notNullValue;
+import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.closeTo;
 import static org.hamcrest.Matchers.greaterThanOrEqualTo;
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.verify;
 
@@ -827,8 +839,8 @@ public class MDLV2000ReaderTest extends SimpleChemObjectReaderTest {
         Assert.assertNotNull(mol);
         Assert.assertEquals(2, mol.getAtom(0).getValency().intValue());
         Assert.assertEquals(3, mol.getAtom(1).getValency().intValue());
-        Assert.assertThat(mol.getAtom(2).getValency(), is(not(0)));
-        Assert.assertThat(mol.getAtom(2).getValency(), is(4));
+        org.hamcrest.MatcherAssert.assertThat(mol.getAtom(2).getValency(), is(not(0)));
+        org.hamcrest.MatcherAssert.assertThat(mol.getAtom(2).getValency(), is(4));
         Assert.assertEquals(0, mol.getAtom(3).getValency().intValue());
     }
 
@@ -972,8 +984,9 @@ public class MDLV2000ReaderTest extends SimpleChemObjectReaderTest {
             if (atom.getSymbol().equals("Ir")) {
                 for (IBond bond : atc.getConnectedBondsList(atom)) {
                     if (bond instanceof QueryBond) {
+                      if (((QueryBond) bond).getExpression().type() == Expr.Type.TRUE) {
                         queryBondCount++;
-                        assertSame(((QueryBond) bond).getExpression().type(), Expr.Type.TRUE);
+                      }
                     }
                 }
             }
@@ -995,8 +1008,9 @@ public class MDLV2000ReaderTest extends SimpleChemObjectReaderTest {
 
         for (IBond bond : atc.bonds()) {
             if (bond instanceof QueryBond) {
-                queryBondCount++;
-                assertSame(((QueryBond) bond).getExpression().type(), Expr.Type.SINGLE_OR_AROMATIC);
+                if (((QueryBond) bond).getExpression().type() == Expr.Type.SINGLE_OR_AROMATIC) {
+                    queryBondCount++;
+                }
             }
         }
         Assert.assertEquals("Expecting six 'query' bond types", 6, queryBondCount);
@@ -1012,7 +1026,9 @@ public class MDLV2000ReaderTest extends SimpleChemObjectReaderTest {
         IAtomContainer molecule = DefaultChemObjectBuilder.getInstance().newInstance(IAtomContainer.class);
         reader.read(molecule);
         reader.close();
-        Assert.assertEquals("R", molecule.getAtom(55).getSymbol());
+        IAtom atom = molecule.getAtom(55);
+        org.hamcrest.MatcherAssert.assertThat(atom, CoreMatchers.<IAtom>instanceOf(IPseudoAtom.class));
+        Assert.assertEquals("R", ((IPseudoAtom)atom).getLabel());
     }
 
     @Test
@@ -1370,6 +1386,17 @@ public class MDLV2000ReaderTest extends SimpleChemObjectReaderTest {
         assertThat(((IPseudoAtom) container.getAtom(7)).getLabel(), is("Protein"));
     }
 
+    @Test
+    public void keepAtomicNumberOfAlias() throws Exception {
+        InputStream in = getClass().getResourceAsStream("/data/mdl/element-with-alias.mol");
+        MDLV2000Reader reader = new MDLV2000Reader(in);
+        IAtomContainer container = reader.read(new AtomContainer());
+        reader.close();
+        assertThat(container.getAtom(6), is(instanceOf(IPseudoAtom.class)));
+        assertThat(((IPseudoAtom) container.getAtom(6)).getLabel(), is("N1"));
+        assertThat(((IPseudoAtom) container.getAtom(6)).getAtomicNumber(), is(7));
+    }
+
     @Test
     public void v2000Version() throws Exception {
         assertThat(MDLV2000Reader.CTabVersion.ofHeader("  5  5  0  0  0  0            999 V2000"),
@@ -1744,7 +1771,7 @@ public class MDLV2000ReaderTest extends SimpleChemObjectReaderTest {
             Assert.assertEquals(2, atom.getMassNumber().intValue());
         }
     }
-	
+
     @Test
     public void testBadAtomCoordinateFormat() throws Exception {
 
@@ -1813,4 +1840,193 @@ public class MDLV2000ReaderTest extends SimpleChemObjectReaderTest {
                        is("Blah"));
         }
     }
+
+
+    @Test public void atomList() throws Exception {
+        try (InputStream in = getClass().getResourceAsStream("query_atomlist.mol");
+             MDLV2000Reader mdlr = new MDLV2000Reader(in)) {
+            IQueryAtomContainer mol       = mdlr.read(new QueryAtomContainer(SilentChemObjectBuilder.getInstance()));
+            IAtom               deref     = AtomRef.deref(mol.getAtom(0));
+            assertThat(deref, CoreMatchers.<IAtom>instanceOf(QueryAtom.class));
+            QueryAtom           queryAtom = (QueryAtom) deref;
+            Expr expr = queryAtom.getExpression();
+            Expr expected = new Expr(Expr.Type.ELEMENT, 9) // F
+                .or(new Expr(Expr.Type.ELEMENT, 7)) // N
+                .or(new Expr(Expr.Type.ELEMENT, 8)); // O
+            assertThat(expr, is(expected));
+        }
+    }
+    @Test public void legacyAtomList() throws Exception {
+        try (InputStream in = getClass().getResourceAsStream("query_legacyatomlist.mol");
+             MDLV2000Reader mdlr = new MDLV2000Reader(in)) {
+            IQueryAtomContainer mol       = mdlr.read(new QueryAtomContainer(SilentChemObjectBuilder.getInstance()));
+            IAtom               deref     = AtomRef.deref(mol.getAtom(0));
+            assertThat(deref, CoreMatchers.<IAtom>instanceOf(QueryAtom.class));
+            QueryAtom           queryAtom = (QueryAtom) deref;
+            Expr expr = queryAtom.getExpression();
+            Expr expected = new Expr(Expr.Type.ELEMENT, 9) // F
+                    .or(new Expr(Expr.Type.ELEMENT, 7)) // N
+                    .or(new Expr(Expr.Type.ELEMENT, 8)); // O
+            assertThat(expr, is(expected));
+        }
+    }
+
+    @Test public void notatomList() throws Exception {
+        try (InputStream in = getClass().getResourceAsStream("query_notatomlist.mol");
+             MDLV2000Reader mdlr = new MDLV2000Reader(in)) {
+            IQueryAtomContainer mol       = mdlr.read(new QueryAtomContainer(SilentChemObjectBuilder.getInstance()));
+            IAtom               deref     = AtomRef.deref(mol.getAtom(0));
+            assertThat(deref, CoreMatchers.<IAtom>instanceOf(QueryAtom.class));
+            QueryAtom           queryAtom = (QueryAtom) deref;
+            Expr expr = queryAtom.getExpression();
+            Expr expected = new Expr(Expr.Type.ELEMENT, 9) // F
+                                   .or(new Expr(Expr.Type.ELEMENT, 7)) // N
+                                   .or(new Expr(Expr.Type.ELEMENT, 8)); // O
+            expected = expected.negate();
+            assertThat(expr, is(expected));
+        }
+    }
+    @Test public void legacynotatomList() throws Exception {
+        try (InputStream in = getClass().getResourceAsStream("query_legacynotatomlist.mol");
+             MDLV2000Reader mdlr = new MDLV2000Reader(in)) {
+            IQueryAtomContainer mol       = mdlr.read(new QueryAtomContainer(SilentChemObjectBuilder.getInstance()));
+            IAtom               deref     = AtomRef.deref(mol.getAtom(0));
+            assertThat(deref, CoreMatchers.<IAtom>instanceOf(QueryAtom.class));
+            QueryAtom           queryAtom = (QueryAtom) deref;
+            Expr expr = queryAtom.getExpression();
+            Expr expected = new Expr(Expr.Type.ELEMENT, 9) // F
+                    .or(new Expr(Expr.Type.ELEMENT, 7)) // N
+                    .or(new Expr(Expr.Type.ELEMENT, 8)); // O
+            expected = expected.negate();
+            assertThat(expr, is(expected));
+        }
+    }
+
+    @Test public void sgroupsAbbrRoundTrip() throws IOException, CDKException {
+        StringWriter sw = new StringWriter();
+        try (InputStream in = getClass().getResourceAsStream("sgroup-sup.mol3");
+             MDLV3000Reader mdlr = new MDLV3000Reader(in);
+             MDLV2000Writer mdlw = new MDLV2000Writer(sw)) {
+            IAtomContainer mol = SilentChemObjectBuilder.getInstance().newAtomContainer();
+            mol = mdlr.read(mol);
+            mdlw.write(mol);
+        }
+        assertThat(sw.toString(), containsString("M  SAL   1  2   2   3"));
+    }
+
+    @Test
+    public void checkFuseBondWithFewerBondsThanAtoms() throws IOException, CDKException {
+        try (InputStream in = getClass().getResourceAsStream("potentialLateFuse.mol");
+            MDLV2000Reader reader = new MDLV2000Reader(in)) {
+            IAtomContainer mol = reader.read(SilentChemObjectBuilder.getInstance().newAtomContainer());
+            assertThat(mol.getAtomCount(), is(108));
+        }
+    }
+
+    @Test
+    public void atomlistWithAtomContainer() throws Exception {
+        try (InputStream in = getClass().getResourceAsStream("query_notatomlist.mol");
+             MDLV2000Reader mdlr = new MDLV2000Reader(in)) {
+
+            IAtomContainer mol   = mdlr.read(SilentChemObjectBuilder.getInstance().newAtomContainer());
+            IAtom          deref = AtomRef.deref(mol.getAtom(0));
+            assertThat(deref, CoreMatchers.<IAtom>instanceOf(QueryAtom.class));
+        }
+    }
+
+    @Test
+    public void dataSgroup() {
+        String path = "/data/mdl/hbr_acoh_mix.mol";
+        try (InputStream in = getClass().getResourceAsStream(path)) {
+            MDLV2000Reader     mdlr     = new MDLV2000Reader(in);
+            IChemObjectBuilder builder = SilentChemObjectBuilder.getInstance();
+            IAtomContainer mol = mdlr.read(builder.newAtomContainer());
+            List<Sgroup> sgroups = mol.getProperty(CDKConstants.CTAB_SGROUPS);
+            Sgroup dataSgroup = null;
+            for (Sgroup sgroup : sgroups) {
+                if (sgroup.getType() == SgroupType.CtabData) {
+                    dataSgroup = sgroup;
+                    break;
+                }
+            }
+            assertNotNull(dataSgroup);
+            assertThat(dataSgroup.<String>getValue(SgroupKey.DataFieldName),
+                       CoreMatchers.is("WEIGHT_PERCENT"));
+            // note it looks like MDL/Accelys/BIOVIA simply omit units/format
+            // but check we pass it okay
+            assertThat(dataSgroup.<String>getValue(SgroupKey.DataFieldUnits),
+                       CoreMatchers.is("%"));
+            assertThat(dataSgroup.<String>getValue(SgroupKey.DataFieldFormat),
+                       CoreMatchers.is("N"));
+            assertThat(dataSgroup.<String>getValue(SgroupKey.Data),
+                       CoreMatchers.is("33%"));
+        } catch (IOException | CDKException e) {
+            e.printStackTrace();
+        }
+    }
+
+    @Test
+    public void testNoChiralFlag() throws Exception {
+        final String input = "\n" +
+                "  Mrv1810 02052112282D          \n" +
+                "\n" +
+                "  7  7  0  0  0  0            999 V2000\n" +
+                "   -1.1468    6.5972    0.0000 C   0  0  2  0  0  0  0  0  0  0  0  0\n" +
+                "   -1.8613    6.1847    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -1.8613    5.3597    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -1.1468    4.9472    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -0.4323    5.3597    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -0.4323    6.1847    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -1.1468    7.4222    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "  1  2  1  0  0  0  0\n" +
+                "  2  3  1  0  0  0  0\n" +
+                "  3  4  1  0  0  0  0\n" +
+                "  4  5  1  0  0  0  0\n" +
+                "  5  6  1  0  0  0  0\n" +
+                "  1  6  1  0  0  0  0\n" +
+                "  1  7  1  1  0  0  0\n" +
+                "M  END\n";
+        IChemObjectBuilder bldr = SilentChemObjectBuilder.getInstance();
+        try (MDLV2000Reader mdlr = new MDLV2000Reader(new StringReader(input))) {
+            IAtomContainer mol = mdlr.read(bldr.newAtomContainer());
+            Iterable<IStereoElement> iter = mol.stereoElements();
+            assertTrue(iter.iterator().hasNext());
+            for (IStereoElement<?,?> se : iter) {
+                assertThat(se.getGroupInfo(), is(IStereoElement.GRP_RAC1));
+            }
+        }
+    }
+
+    @Test
+    public void testChiralFlag() throws Exception {
+        final String input = "\n" +
+                "  Mrv1810 02052112282D          \n" +
+                "\n" +
+                "  7  7  0  0  1  0            999 V2000\n" +
+                "   -1.1468    6.5972    0.0000 C   0  0  2  0  0  0  0  0  0  0  0  0\n" +
+                "   -1.8613    6.1847    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -1.8613    5.3597    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -1.1468    4.9472    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -0.4323    5.3597    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -0.4323    6.1847    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "   -1.1468    7.4222    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0\n" +
+                "  1  2  1  0  0  0  0\n" +
+                "  2  3  1  0  0  0  0\n" +
+                "  3  4  1  0  0  0  0\n" +
+                "  4  5  1  0  0  0  0\n" +
+                "  5  6  1  0  0  0  0\n" +
+                "  1  6  1  0  0  0  0\n" +
+                "  1  7  1  1  0  0  0\n" +
+                "M  END\n";
+        IChemObjectBuilder bldr = SilentChemObjectBuilder.getInstance();
+        try (MDLV2000Reader mdlr = new MDLV2000Reader(new StringReader(input))) {
+            IAtomContainer mol = mdlr.read(bldr.newAtomContainer());
+            Iterable<IStereoElement> iter = mol.stereoElements();
+            assertTrue(iter.iterator().hasNext());
+            for (IStereoElement<?,?> se : iter) {
+                // Grp Abs is actually just 0
+                assertThat(se.getGroupInfo(), is(IStereoElement.GRP_ABS));
+            }
+        }
+    }
 }
