@@ -140,7 +140,7 @@ public class InChIToStructureTest extends CDKTestCase {
         IAtomContainer container = parser.getAtomContainer();
         // test if the created IAtomContainer is done with the Silent module...
         // OK, this is not typical use, but maybe the above generate method should be private
-        Assert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
+        org.hamcrest.MatcherAssert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
     }
 
     @Test
@@ -148,9 +148,9 @@ public class InChIToStructureTest extends CDKTestCase {
         InChIToStructure parser = new InChIToStructure("InChI=1S/O", DefaultChemObjectBuilder.getInstance());
         parser.generateAtomContainerFromInchi(SilentChemObjectBuilder.getInstance());
         IAtomContainer container = parser.getAtomContainer();
-        Assert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
-        Assert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(notNullValue()));
-        Assert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(0));
+        org.hamcrest.MatcherAssert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
+        org.hamcrest.MatcherAssert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(notNullValue()));
+        org.hamcrest.MatcherAssert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(0));
     }
 
     @Test
@@ -158,10 +158,10 @@ public class InChIToStructureTest extends CDKTestCase {
         InChIToStructure parser = new InChIToStructure("InChI=1S/H2O/h1H2/i1+2", DefaultChemObjectBuilder.getInstance());
         parser.generateAtomContainerFromInchi(SilentChemObjectBuilder.getInstance());
         IAtomContainer container = parser.getAtomContainer();
-        Assert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(notNullValue()));
-        Assert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));        Assert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(notNullValue()));
-        Assert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(2));
-        Assert.assertThat(container.getAtom(0).getMassNumber(), is(18));
+        org.hamcrest.MatcherAssert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(notNullValue()));
+        org.hamcrest.MatcherAssert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));        org.hamcrest.MatcherAssert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(notNullValue()));
+        org.hamcrest.MatcherAssert.assertThat(container.getAtom(0).getImplicitHydrogenCount(), is(2));
+        org.hamcrest.MatcherAssert.assertThat(container.getAtom(0).getMassNumber(), is(18));
     }
 
     @Test
@@ -171,7 +171,7 @@ public class InChIToStructureTest extends CDKTestCase {
         parser.generateAtomContainerFromInchi(SilentChemObjectBuilder.getInstance());
         IAtomContainer container = parser.getAtomContainer();
         Iterator<IStereoElement> ses = container.stereoElements().iterator();
-        Assert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
+        org.hamcrest.MatcherAssert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
         assertTrue(ses.hasNext());
         IStereoElement se = ses.next();
         assertThat(se, is(instanceOf(IDoubleBondStereochemistry.class)));
@@ -185,7 +185,7 @@ public class InChIToStructureTest extends CDKTestCase {
         parser.generateAtomContainerFromInchi(SilentChemObjectBuilder.getInstance());
         IAtomContainer container = parser.getAtomContainer();
         Iterator<IStereoElement> ses = container.stereoElements().iterator();
-        Assert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
+        org.hamcrest.MatcherAssert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
         assertTrue(ses.hasNext());
         IStereoElement se = ses.next();
         assertThat(se, is(instanceOf(IDoubleBondStereochemistry.class)));
@@ -202,7 +202,7 @@ public class InChIToStructureTest extends CDKTestCase {
         IAtomContainer container = parser.getAtomContainer();
 
         Iterator<IStereoElement> ses = container.stereoElements().iterator();
-        Assert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
+        org.hamcrest.MatcherAssert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
         assertTrue(ses.hasNext());
         IStereoElement se = ses.next();
         assertThat(se, is(instanceOf(ExtendedTetrahedral.class)));
@@ -223,7 +223,7 @@ public class InChIToStructureTest extends CDKTestCase {
         IAtomContainer container = parser.getAtomContainer();
 
         Iterator<IStereoElement> ses = container.stereoElements().iterator();
-        Assert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
+        org.hamcrest.MatcherAssert.assertThat(container, is(instanceOf(SilentChemObjectBuilder.getInstance().newAtomContainer().getClass())));
         assertTrue(ses.hasNext());
         IStereoElement se = ses.next();
         assertThat(se, is(instanceOf(ExtendedTetrahedral.class)));
@@ -256,4 +256,13 @@ public class InChIToStructureTest extends CDKTestCase {
         }
         assertThat(dCount, is(5));
     }
+
+    @Test
+    public void isotope() throws Exception{
+        InChIToStructure parse = new InChIToStructure("InChI=1S/HI/h1H/p-1/i1-6",
+                SilentChemObjectBuilder.getInstance());
+        IAtomContainer mol = parse.getAtomContainer();
+        assertThat(mol.getAtomCount(), is(1));
+        assertThat(mol.getAtom(0).getMassNumber(), is(121));
+    }
 }
